@@ -27,6 +27,8 @@ typedef struct tpm_rsa_key	tpm_rsa_key_t;
 
 extern tpm_rsa_key_t *	tpm_rsa_key_read_public(const char *pathname);
 extern tpm_rsa_key_t *	tpm_rsa_key_read_private(const char *pathname);
+extern bool		tpm_rsa_key_write_public(const char *pathname,
+				const tpm_rsa_key_t *key);
 extern bool		tpm_rsa_key_write_private(const char *pathname,
 				const tpm_rsa_key_t *key);
 extern void		tpm_rsa_key_free(tpm_rsa_key_t *key);
@@ -37,4 +39,6 @@ extern int		tpm_rsa_sign(const tpm_rsa_key_t *,
 
 extern TPM2B_PUBLIC *	tpm_rsa_key_to_tss2(const tpm_rsa_key_t *key);
 
+extern const tpm_evdigest_t * tpm_rsa_key_public_digest(const tpm_rsa_key_t *pubkey);
+
 #endif /* RSA_H */
