@@ -42,6 +42,7 @@ struct testcase {
 	char *			partition_directory;
 	char *			disk_directory;
 	char *			hash_log;
+	char *			prep_partition;
 
 	FILE *			hash_log_fp;
 };
@@ -189,7 +190,8 @@ testcase_read_symlink(const char *directory, const char *name, const char *defau
 		fatal("Cannot read symlink %s: %m\n", path);
 
 	if (target[0] != '/' && default_dir) {
-		snprintf(result, sizeof(result), "%s/%s", default_dir, target);
+		if (snprintf(result, sizeof(result), "%s/%s", default_dir, target) < 0)
+			return NULL;
 		return strdup(result);
 	}
 
@@ -224,12 +226,18 @@ testcase_write_file(const char *directory, const char *name, const buffer_t *bp)
 }
 
 static buffer_t *
-testcase_read_file(const char *directory, const char *name)
+__testcase_read_file(const char *directory, const char *name, int flags)
 {
 	char path[PATH_MAX];
 
 	snprintf(path, sizeof(path), "%s/%s", directory, name);
-	return runtime_read_file(path, 0);
+	return runtime_read_file(path, flags);
+}
+
+static buffer_t *
+testcase_read_file(const char *directory, const char *name)
+{
+	return __testcase_read_file(directory, name, 0);
 }
 
 testcase_t *
@@ -314,7 +322,12 @@ testcase_record_efi_variable(testcase_t *tc, const char *name, const buffer_t *d
 buffer_t *
 testcase_playback_efi_variable(testcase_t *tc, const char *name)
 {
-	return testcase_read_file(tc->efi_directory, name);
+	/* For systems in UEFI Setup mode, there is no PK, KEK, or db, but those
+	 * variables are still recorded in the TPM event log with zero length.
+	 * Set the file reading flag to skip those EFI variable files.
+	 */
+	return __testcase_read_file(tc->efi_directory, name,
+				    RUNTIME_MISSING_FILE_OKAY);
 }
 
 void
@@ -588,3 +601,38 @@ testcase_playback_efi_digest(testcase_t *tc, const char *path, const tpm_algo_in
 {
 	return testcase_playback_digest(tc, "efi", path, algo);
 }
+
+void
+testcase_record_prep_partition(testcase_t *tc, const char *path)
+{
+	int fd;
+
+	if ((fd = testcase_create_file(tc->base_directory, "prep_partition")) < 0)
+		fatal("Failed to create PReP partition recording file: %m\n");
+
+	if (write(fd, path, strlen(path) + 1) < 0)
+		fatal("Failed to record PReP partition: %m\n");
+
+	close(fd);
+}
+
+char *
+testcase_playback_prep_partition(testcase_t *tc)
+{
+	int fd;
+	char buf[PATH_MAX];
+	ssize_t count;
+
+	if ((fd = testcase_open_file(tc->base_directory, "prep_partition")) < 0)
+		return NULL;
+
+	count = read(fd, buf, PATH_MAX - 1);
+	if (count < 0)
+		fatal("failed to read prep_partition: %m\n");
+
+	buf[count+1] = '\0';
+
+	close(fd);
+
+	return strdup(buf);
+}
